﻿// ReSharper disable MemberCanBeInternal
// ReSharper disable MemberCanBePrivate.Global
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Класс сообщения состояния
/// </summary>
public sealed class ActionStateMessage: ISerializable
{
    #region Свойства
    /// <summary>
    /// Критичность сообщения
    /// </summary>
    public bool IsCritical { get; set; }

    /// <summary>
    /// Объект сообщения
    /// </summary>
    public object Object { get; set; }

    /// <summary>
    /// Текст сообщения
    /// </summary>
    public string Message { get; set; }
    #endregion

    #region Конструкторы
    /// <summary>
    /// Конструктор по умолчанию
    /// </summary>
    // ReSharper disable once MemberCanBePrivate.Global
    public ActionStateMessage ()
    {
        IsCritical = true;
        Object = "Not Implemented";
        Message = "Not Implemented";
    }

    /// <summary>
    /// Конструктор с 2 параметрами
    /// </summary>
    /// <param name="message">Сообщение</param>
    /// <param name="isCritical">Критичность сообщения</param>
    public ActionStateMessage (string message, bool isCritical = true)
    {
        IsCritical = isCritical;
        Object = "";
        Message = message;
    }

    /// <summary>
    /// Конструктор с 3 параметрами
    /// </summary>
    /// <param name="eObject">Объект сообщения</param>
    /// <param name="message">Сообщение</param>
    /// <param name="isCritical">Критичность сообщения</param>
    public ActionStateMessage (object eObject, string message, bool isCritical = true)
    {
        IsCritical = isCritical;
        Object = eObject;
        Message = message;
    }
    #endregion

    #region Методы
    /// <summary>
    /// Вывод сообщения
    /// </summary>
    /// <param name="format">Строка-форматирование (например, «[{0}] - {1}»)</param>
    /// <returns>Отформатированную строка</returns>
    public string PrintMessage (string format) => string.Format(format, Object, Message);
    #endregion

    #region Реализация интерфейса ISerializable
    /// <inheritdoc />
    public string Serialize () => new NewtonsoftJsonSerializer().Serialize(this);

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Десериализую строку
        ActionStateMessage item = new NewtonsoftJsonSerializer().Deserialize<ActionStateMessage>(json) ?? new();

        // Передаю параметры
        IsCritical = item.IsCritical;
        Object = item.Object;
        Message = item.Message;
    }
    #endregion
}