﻿using anbs_cp.Enums;

namespace anbs_cp.Exceptions;

/// <summary>
/// Ошибка (де)сериализации
/// </summary>
/// <param name="action">Действие сериализатора, приведшее к ошибке</param>
/// <param name="objectJson">Объект сериализации при сериализации или строка json при десериализации</param>
/// <param name="message">Сообщение пользователю</param>
public sealed class SerializeException (ESerializeExceptionAction action, string? objectJson, string? message = null)
    : Exception(message)
{
    /// <summary>
    /// Действие сериализатора, приведшее к ошибке
    /// </summary>
    public ESerializeExceptionAction Action => action;

    /// <summary>
    /// Объект сериализации (при action == ESerializeExceptionAction.Serialize)
    /// </summary>
    public string? Object => action == ESerializeExceptionAction.Serialize ? objectJson : null;

    /// <summary>
    /// Строка json при десериализации
    /// </summary>
    public string? Json => action == ESerializeExceptionAction.Deserialize ? objectJson : null;
}